@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
     babel.core
     ~~~~~~~~~~
@@ -105,7 +104,7 @@ class UnknownLocaleError(Exception):
         self.identifier = identifier
 
 
-class Locale(object):
+class Locale:
     """Representation of a specific locale.
 
     >>> locale = Locale('en', 'US')
@@ -565,6 +564,18 @@ class Locale(object):
         """
         return self._data['decimal_formats']
 
+    @property
+    def compact_decimal_formats(self):
+        """Locale patterns for compact decimal number formatting.
+
+        .. note:: The format of the value returned may change between
+                  Babel versions.
+
+        >>> Locale('en', 'US').compact_decimal_formats["short"]["one"]["1000"]
+        <NumberPattern u'0K'>
+        """
+        return self._data['compact_decimal_formats']
+
     @property
     def currency_formats(self):
         """Locale patterns for currency number formatting.
@@ -999,7 +1010,7 @@ def negotiate_locale(preferred, available, sep='_', aliases=LOCALE_ALIASES):
     user to easily add the territory. So while you may prefer using qualified
     locale identifiers in your web-application, they would not normally match
     the language-only locale sent by such browsers. To workaround that, this
-    function uses a default mapping of commonly used langauge-only locale
+    function uses a default mapping of commonly used language-only locale
     identifiers to identifiers including the territory:
 
     >>> negotiate_locale(['ja', 'en_US'], ['ja_JP', 'en_US'])
