@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
     babel.dates
     ~~~~~~~~~~~
@@ -16,7 +15,6 @@
     :license: BSD, see LICENSE for more details.
 """
 
-from __future__ import division
 
 import re
 import warnings
@@ -262,7 +260,7 @@ def get_next_timezone_transition(zone=None, dt=None):
     )
 
 
-class TimezoneTransition(object):
+class TimezoneTransition:
     """A helper object that represents the return value from
     :func:`get_next_timezone_transition`.
 
@@ -1113,6 +1111,12 @@ def get_period_id(time, tzinfo=None, type=None, locale=LC_TIME):
     >>> get_period_names(locale="de")[get_period_id(time(7, 42), locale="de")]
     u'Morgen'
 
+    >>> get_period_id(time(0), locale="en_US")
+    u'midnight'
+
+    >>> get_period_id(time(0), type="selection", locale="en_US")
+    u'night1'
+
     :param time: The time to inspect.
     :param tzinfo: The timezone for the time. See ``format_time``.
     :param type: The period type to use. Either "selection" or None.
@@ -1136,6 +1140,16 @@ def get_period_id(time, tzinfo=None, type=None, locale=LC_TIME):
 
     for rule_id, rules in rulesets:
         for rule in rules:
+            if "from" in rule and "before" in rule:
+                if rule["from"] < rule["before"]:
+                    if rule["from"] <= seconds_past_midnight < rule["before"]:
+                        return rule_id
+                else:
+                    # e.g. from="21:00" before="06:00"
+                    if rule["from"] <= seconds_past_midnight < 86400 or \
+                            0 <= seconds_past_midnight < rule["before"]:
+                        return rule_id
+
             start_ok = end_ok = False
 
             if "from" in rule and seconds_past_midnight >= rule["from"]:
@@ -1165,13 +1179,18 @@ class ParseError(ValueError):
 def parse_date(string, locale=LC_TIME, format='medium'):
     """Parse a date from a string.
 
-    This function uses the date format for the locale as a hint to determine
-    the order in which the date fields appear in the string.
+    This function first tries to interpret the string as ISO-8601
+    date format, then uses the date format for the locale as a hint to
+    determine the order in which the date fields appear in the string.
 
     >>> parse_date('4/1/04', locale='en_US')
     datetime.date(2004, 4, 1)
     >>> parse_date('01.04.2004', locale='de_DE')
     datetime.date(2004, 4, 1)
+    >>> parse_date('2004-04-01', locale='en_US')
+    datetime.date(2004, 4, 1)
+    >>> parse_date('2004-04-01', locale='de_DE')
+    datetime.date(2004, 4, 1)
 
     :param string: the string containing the date
     :param locale: a `Locale` object or a locale identifier
@@ -1181,7 +1200,16 @@ def parse_date(string, locale=LC_TIME, format='medium'):
     if not numbers:
         raise ParseError("No numbers were found in input")
 
-    # TODO: try ISO format first?
+    # we try ISO-8601 format first, meaning similar to formats
+    # extended YYYY-MM-DD or basic YYYYMMDD
+    iso_alike = re.match(r'^(\d{4})-?([01]\d)-?([0-3]\d)$',
+                         string, flags=re.ASCII)  # allow only ASCII digits
+    if iso_alike:
+        try:
+            return date(*map(int, iso_alike.groups()))
+        except ValueError:
+            pass  # a locale format might fit better, so let's continue
+
     format_str = get_date_format(format=format, locale=locale).pattern.lower()
     year_idx = format_str.index('y')
     month_idx = format_str.index('m')
@@ -1191,7 +1219,7 @@ def parse_date(string, locale=LC_TIME, format='medium'):
 
     indexes = [(year_idx, 'Y'), (month_idx, 'M'), (day_idx, 'D')]
     indexes.sort()
-    indexes = dict([(item[1], idx) for idx, item in enumerate(indexes)])
+    indexes = {item[1]: idx for idx, item in enumerate(indexes)}
 
     # FIXME: this currently only supports numbers, but should also support month
     #        names, both in the requested locale, and english
@@ -1237,7 +1265,7 @@ def parse_time(string, locale=LC_TIME, format='medium'):
 
     indexes = [(hour_idx, 'H'), (min_idx, 'M'), (sec_idx, 'S')]
     indexes.sort()
-    indexes = dict([(item[1], idx) for idx, item in enumerate(indexes)])
+    indexes = {item[1]: idx for idx, item in enumerate(indexes)}
 
     # TODO: support time zones
 
@@ -1258,7 +1286,7 @@ def parse_time(string, locale=LC_TIME, format='medium'):
     return time(hour, minute, second)
 
 
-class DateTimePattern(object):
+class DateTimePattern:
 
     def __init__(self, pattern, format):
         self.pattern = pattern
@@ -1267,9 +1295,6 @@ class DateTimePattern(object):
     def __repr__(self):
         return '<%s %r>' % (type(self).__name__, self.pattern)
 
-    def __unicode__(self):
-        return self.pattern
-
     def __str__(self):
         pat = self.pattern
         return pat
@@ -1283,7 +1308,7 @@ class DateTimePattern(object):
         return self % DateTimeFormat(datetime, locale)
 
 
-class DateTimeFormat(object):
+class DateTimeFormat:
 
     def __init__(self, value, locale):
         assert isinstance(value, (date, datetime, time))
@@ -1313,9 +1338,8 @@ class DateTimeFormat(object):
             return self.format_day_of_week_in_month()
         elif char in ('E', 'e', 'c'):
             return self.format_weekday(char, num)
-        elif char == 'a':
-            # TODO: Add support for the rest of the period formats (a*, b*, B*)
-            return self.format_period(char)
+        elif char in ('a', 'b', 'B'):
+            return self.format_period(char, num)
         elif char == 'h':
             if self.value.hour % 12 == 0:
                 return self.format(12, num)
@@ -1453,10 +1477,44 @@ class DateTimeFormat(object):
     def format_day_of_week_in_month(self):
         return '%d' % ((self.value.day - 1) // 7 + 1)
 
-    def format_period(self, char):
-        period = {0: 'am', 1: 'pm'}[int(self.value.hour >= 12)]
-        for width in ('wide', 'narrow', 'abbreviated'):
-            period_names = get_period_names(context='format', width=width, locale=self.locale)
+    def format_period(self, char, num):
+        """
+        Return period from parsed datetime according to format pattern.
+
+        >>> format = DateTimeFormat(time(13, 42), 'fi_FI')
+        >>> format.format_period('a', 1)
+        u'ip.'
+        >>> format.format_period('b', 1)
+        u'iltap.'
+        >>> format.format_period('b', 4)
+        u'iltapäivä'
+        >>> format.format_period('B', 4)
+        u'iltapäivällä'
+        >>> format.format_period('B', 5)
+        u'ip.'
+
+        >>> format = DateTimeFormat(datetime(2022, 4, 28, 6, 27), 'zh_Hant')
+        >>> format.format_period('a', 1)
+        u'上午'
+        >>> format.format_period('b', 1)
+        u'清晨'
+        >>> format.format_period('B', 1)
+        u'清晨'
+
+        :param char: pattern format character ('a', 'b', 'B')
+        :param num: count of format character
+
+        """
+        widths = [{3: 'abbreviated', 4: 'wide', 5: 'narrow'}[max(3, num)],
+                  'wide', 'narrow', 'abbreviated']
+        if char == 'a':
+            period = 'pm' if self.value.hour >= 12 else 'am'
+            context = 'format'
+        else:
+            period = get_period_id(self.value, locale=self.locale)
+            context = 'format' if char == 'B' else 'stand-alone'
+        for width in widths:
+            period_names = get_period_names(context=context, width=width, locale=self.locale)
             if period in period_names:
                 return period_names[period]
         raise ValueError('Could not format period %s in %s' % (period, self.locale))
@@ -1579,7 +1637,7 @@ PATTERN_CHARS = {
     'w': [1, 2], 'W': [1],                                              # week
     'd': [1, 2], 'D': [1, 2, 3], 'F': [1], 'g': None,                   # day
     'E': [1, 2, 3, 4, 5, 6], 'e': [1, 2, 3, 4, 5, 6], 'c': [1, 3, 4, 5, 6],  # week day
-    'a': [1],                                                           # period
+    'a': [1, 2, 3, 4, 5], 'b': [1, 2, 3, 4, 5], 'B': [1, 2, 3, 4, 5],   # period
     'h': [1, 2], 'H': [1, 2], 'K': [1, 2], 'k': [1, 2],                 # hour
     'm': [1, 2],                                                        # minute
     's': [1, 2], 'S': None, 'A': None,                                  # second
